import logging

class Colors():
    grey = '\x1b[38;21m'
    blue = '\x1b[38;5;39m'
    yellow = '\x1b[38;5;226m'
    red = '\x1b[38;5;196m'
    bold_red = '\x1b[31;1m'
    reset = '\x1b[0m'

    bgblack = "\x1b[40m"
    bgred = "\x1b[41m"
    bggreen = "\x1b[42m"
    bgyellow = "\x1b[43m"
    bgblue = "\x1b[44m"
    bgmagenta = "\x1b[45m"
    bgcyan = "\x1b[46m"
    bgwhite = "\x1b[47m"

    fgblack = "\x1b[30m"
    fgred = "\x1b[31m"
    fggreen = "\x1b[32m"
    fgyellow = "\x1b[33m"
    fgblue = "\x1b[34m"
    fgmagenta = "\x1b[35m"
    fgcyan = "\x1b[36m"
    fgwhite = "\x1b[37m"

class CustomFormatter(logging.Formatter):

    grey = "\x1b[38;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"

    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: grey + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)



from urllib.request import urlopen, Request  ### from urllib.parse import urlencode
from urllib.error import HTTPError
import json


def request(url, data={}, headers={}, method="POST"):
    rq = Request(url, json.dumps(data).encode("ascii"), headers, method=method)
    data = {}
    try:
        response = urlopen(rq)
        data = json.load(response)
    except HTTPError as e:
        raise e
    return data

def which(program):
    def is_exe(fpath):
        return os.path.exists(fpath) and os.access(fpath, os.X_OK) and os.path.isfile(fpath)

    def ext_candidates(fpath):
        yield fpath
        for ext in os.environ.get("PATHEXT", "").split(os.pathsep):
            yield fpath + ext

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            for candidate in ext_candidates(exe_file):
                if is_exe(candidate):
                    return candidate

    return None



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
        name='daemondev_config_server',
        version='0.5',
        author='@daemondev',
        author_email='granlinux@gmail.com',
        description='Config-Files downloader',
        long_description=long_description,
        license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
        url = 'https://github.com/daemondev/daemondev-config-server',   # Provide either the link to your github or to your website
        long_description_content_type='text/markdown',
        #packages=setuptools.find_packages(),
        #packages = ['src/config_server'],   # Chose the same as "name"
        package_dir={"": "src", "core": "src/config_server/core"},
        include_package_data=True,
        packages=(
            setuptools.find_packages(where="src") +
            setuptools.find_packages(include=('colors*.py',"core")) +
            setuptools.find_packages(where="src/config_server", )
            ),
        classifiers=[
                'Programming Language :: Python :: 3',
                'License :: OSI Approved :: MIT License',
                'Operating System :: OS Independent',
                ],
        python_requires='>=3.6',
        install_requires=["pyyaml"],
        )



from core.request import request
import os
import base64
import logging
import sys
import yaml
import re
from colors import Colors as colors   ### Run in local
from attribute_dict import AttributeDict
#from config_server.colors import Colors as colors  ### Run installed

#import colors as C
#colors = C.Colors

#from dotenv import load_dotenv
#load_dotenv()

logging.basicConfig(filename='/tmp/daemondev-config-server.log', format='%(asctime)s-daemondev-config-server> %(levelname)s-%(message)s', level=logging.INFO)
_logger = logging.getLogger(__name__)
h = logging.StreamHandler(sys.stdout)
h.flush = sys.stdout.flush
h.setLevel(logging.DEBUG)                   #h.setFormatter(CustomFormatter())
_logger.addHandler(h)

K8S = os.getenv("KUBERNETES_PORT", None )
envConfig = f"{os.getcwd()}/.configserver"

def readfile(filename, is_yaml=False):
    content = ""
    with open(filename, "r") as f:
        if is_yaml:
            content = yaml.load(f, Loader=yaml.FullLoader)
        else:
            content = f.read().strip()
    return content

if os.path.exists(envConfig):
    envs = readfile(envConfig)
    for l in envs.split("\n"):
        key, val = l.split("=")
        os.environ[key.strip()] = val.strip()

configFilesDir = f"{os.path.dirname(os.path.realpath(__file__))}"
repo_name = os.path.basename(os.popen("git rev-parse --show-toplevel").read().strip())
environment = os.getenv("ENVIRONMENT", "local")
debug = os.getenv("DEBUG", False)
config_files_folder = os.getenv("LOCAL_CONFIG_DIR", os.getenv("GLOBAL_CONFIG_DIR", configFilesDir))
allowed_debugable_environmets = ["local", "development", "staging"]

schema = os.getenv("CONFIG_SCHEMA", "https")
config_server = f"{os.getenv('CONFIG_SERVER','config.wecourier4u.com')}"
obj = None
final_local_yaml = ""
project = os.getenv("APPLICATION", os.getenv("PROJECT_NAME", repo_name or "dummy"))
yamlfile = f"{config_files_folder}/{project}-{environment}"
check_env_var_mode_re = r"\${.+}"
inner_content_re = r"(?<=\${)[^{\}].+(?=})"
executionMessage = ""

if os.path.exists(configFilesDir) and os.getcwd() == "/app":
    executionMessage = f"{colors.bggreen}{colors.fgblack}>>> Running in {colors.bgmagenta}\"docker-mode\"{colors.reset}"

if environment in allowed_debugable_environmets and debug:
    ENV_VARS=f" APPLICATION: {project}\n ENVIRONMENT: {environment}\n LOCAL_CONFIG_DIR: {config_files_folder}"
    _logger.info(f"{colors.bgmagenta}{colors.fgblack}ENV VARS:{executionMessage if executionMessage != '' else ''}{colors.reset}{colors.bgyellow}\n{ENV_VARS}{colors.reset}")

def interpolate(obj, key):
    final = obj[key]
    if re.match(check_env_var_mode_re, final):
        env_var_name = re.findall(inner_content_re, final)
        _default = ""
        if "," in final:
            splitted = env_var_name[0].split(",")
            env_var_name = splitted[0].strip()
            _default = splitted[1].strip()
        final = f"{os.getenv(env_var_name, _default)}"
    return final

def iterator(obj):
    for k in obj.keys():
        if isinstance(obj[k], dict):
            iterator(obj[k])
        elif isinstance(obj[k], str):
            obj[k] = interpolate(obj, k)

if K8S is None:
    schema = "http"
    config_server = "localhost:5000"

for ext in ["yaml", "yml"]:
    final_local_yaml =  f"{yamlfile}.{ext}"
    if os.path.exists(final_local_yaml):
        obj = readfile(final_local_yaml, True)

if obj is None:
    auth = f"{os.getenv('CONFIG_USERNAME', 'w4u')}:{os.getenv('CONFIG_PASSWORD','w4u')}".encode("ascii")
    auth = base64.b64encode(auth).decode("utf-8")
    url = f"{schema}://{config_server}/{project}-{environment}"

    custom_headers = {
        "Content-Type": "application/json",
        'Authorization': f"Bearer {auth}"
    }
    obj = request(url, {}, custom_headers, method="GET")
    _logger.info(f"{colors.bggreen}>>> Successful load from request: {colors.bgmagenta}[{url}]!!!{colors.reset}")
else:
    _logger.info(f"{colors.bggreen}{colors.fgblack}>>> Successful load from local directory: {colors.bgmagenta}[{final_local_yaml}]{colors.reset}")


if debug and environment in allowed_debugable_environmets:
    import json
    _logger.info(f"{colors.bgblue}")
    _logger.info(json.dumps(obj,  indent=4, sort_keys=True))
    _logger.info(f"{colors.reset}")

iterator(obj)

def objectify(obj):
    for k, v in obj.items():
        if isinstance(v, dict):
            obj[k] = objectify(v)
    return AttributeDict(obj)

config = objectify(obj)

#print(f">>> obj: [{config.database.name}]")
